/* -*- Mode: C++ -*- */

#ifndef _RULE_STORAGE_H_
#define _RULE_STORAGE_H_

#include <string>
#include <map>

/* This class stores a mapping of string to a pointer to an object that you
   specify.
   Use it like this, if you want to store pointers to Foo
   RuleStorage<Foo> myrulesotrage
   This class will NOT take over the memory of anything which you put into it
*/

template <class Rule>
class RuleStorage
{
public:
  RuleStorage() : storage() {}
  ~RuleStorage() {}

  // returns the Rule which used to have this name, or NULL if this is a new name
  Rule*
  insert(const std::string& name, Rule* p)
  {
    Storage::iterator iter = storage.find(name);
    if (iter == storage.end())
      {
	storage[name] = p;
	return NULL;
      }
    Rule* old = iter->second;
    iter->second = p;
    return old;
  }

  // Remove the rule with the given name
  // Return the Rule, or NULL if the name did not exist
  Rule*
  erase(const std::string& name)
  {
    Storage::iterator iter = storage.find(name);
    if (iter == storage.end())
      {
	return NULL;
      }
    Rule* old = iter->second;
    storage.erase(iter);
    return old;
  }

  // Lookups the rule with the given name
  // Return NULL if no such rule defined
  Rule*
  lookup(const std::string& name)
  {
    Storage::iterator iter = storage.find(name);
    if (iter == storage.end())
      {
	return NULL;
      }
    return iter->second;
  }

  // Lookups the rule with the given name
  // Return NULL if no such rule defined
  // const variant of above
  const Rule*
  lookup(const std::string& name) const
  {
    Storage::const_iterator iter = storage.find(name);
    if (iter == storage.end())
      {
	return NULL;
      }
    return iter->second;
  }

  void clear() 
  {
    storage.clear();
  }

  unsigned size()
  {
    return storage.size();
  }

  unsigned empty()
  {
    return storage.empty();
  }
  
private:
  typedef std::map<std::string, Rule*> Storage;

  Storage storage;
};



#endif
