/*
 * ColorTableItem subclasses QTableItem in order to allow us to set 
 *  foreground and background colors on table cells. 
 *
 * Author: John Davin
 * Based on code from Ivica Bukvic at: 
 * http://lists.trolltech.com/qt-interest/2003-02/thread01139-0.html
 */


#include "ColorTableItem.h"



ColorTableItem::ColorTableItem(QTable *t, EditType et, const QString txt, QColor fg, QColor bg)
  : QTableItem( t, et, txt )
{
    fgcolor = fg;
    bgcolor = bg;
}

ColorTableItem::~ColorTableItem()
{

}

void ColorTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bool selected )
{
    QColorGroup g(cg);

    // only set color if the color is valid. (if it was not set, it will not be valid)
    if(bgcolor.isValid()) g.setColor( QColorGroup::Base, bgcolor );
    if(fgcolor.isValid()) g.setColor( QColorGroup::Text, fgcolor );
    QTableItem::paint(p,g,cr,selected);
}
