#!/usr/local/bin/perl5 -w

# This script is to run the coach on a set of logfiles.
# You'll have to hack the script to make it do what you want, but it would be fairly easy
# All the standard configuration stuff is at the top
# The script will also put a copy of itself in the output dir so that you know how it was run
# The arguments should be the logfiles to run on


use File::Copy;
use File::Basename;
use Cwd qw(abs_path);

$usage = "$0 <outputdir> <list of logfiles>";

## Basic stuff I'll need
$coachdir = abs_path(dirname($0)."/../coach");

## Main parameters
# The standard argument of --file coach.conf, run_mode, logfile_fn do not need to be included
# The following replacements are made:
# %FN : the filename of the log we are analyzing
# %FB : just the 'basename' (no leading directory) of the log
# %OD : the output directory
# %ID : a unique ID for each logfile
# Note that we will run in the coach directory!
#$args = '--abstract_track_po_per_cycle_fn %OD/%FB_pocycle.log ';
#$args .= '--abstract_track_po_summ_fn %OD/%FB_posumm.log ';
#$args .= '--abstract_track_po_write_per_cycle off ';
$args = '--file all_off.conf --use_mod_features --use_mod_abstract ';
$args .= '--abstract_state_track_write_per_cycle on  ';
$args .= '--abstract_state_track_per_cycle_fn %OD/%FB_%ID_absstate_cycle_%S.log ';
$args .= '--abstract_state_track_write_summ on ';
$args .= '--abstract_state_track_summ_fn %OD/%FB_%ID_absstate_summ_%S.log ';
$args .= '--abstract_state_track_both_teams on ';

## Sanity checks
(@ARGV != 0) || die $usage;
$outputdir = shift @ARGV;
(-e $outputdir) || die "Output dir '$outputdir' does not exist";
$outputdir = abs_path($outputdir);

## Write some config stuff to the output dir
copy ($0, $outputdir) || die "Could not copy script to outputdir '$outputdir': $!";
copy ("$coachdir/coach.conf", $outputdir) || die "Could not copy coach.conf to outputdir '$outputdir': $!";
open (LOGFILE_LIST_FH, ">$outputdir/LOGFILE_LIST") || die "Could not open logfile list: $!";
print LOGFILE_LIST_FH (join "\n", @ARGV);
print LOGFILE_LIST_FH "\n";
close (LOGFILE_LIST_FH);

open (COMMANDS_LIST_FH, ">$outputdir/COMMANDS") || die "Could not open commands list: $!";

$outputfn = "$outputdir/OUTPUT.LOG";

if (-e $outputfn)
  {
    unlink ($outputfn) || die "Could not remove old output file '$outputfn': $!";
  }

## Now on to the main loop
chdir ($coachdir) || die "Could not chage to coach dir '$coachdir': $!";
$logfile_idx = 0;
foreach $logfn (@ARGV)
  {
    $basefn = basename($logfn);
    $standard_args = "--file coach.conf --logfile_dir $outputdir --action_log_fn ${basefn}_${logfile_idx}_actions.log ";
    $standard_args .= "--run_mode logfile --logfile_fn $logfn ";
    $this_args = $args;
    $this_args =~ s/\%FN/$logfn/g;
    $this_args =~ s/\%FB/$basefn/g;
    $this_args =~ s/\%OD/$outputdir/g;
    $this_args =~ s/\%ID/$logfile_idx/g;

    $full_cmd = "./coach $standard_args $this_args 2>&1 | tee -a $outputfn";
    print COMMANDS_LIST_FH "$full_cmd\n";
    system $full_cmd;

    $logfile_idx++;
  }
