/*
 * CoachableAgentComm - class that is used by the coachable agent to communicate 
 *  with other agents through say messages. 
 *  Uses the coachable agent say message library (saymsglib) to encode and decode
 *  messages.
 *
 *
 * Creation Date: 12/3/03
 * Author: John Davin
 *
 */


#ifndef COACHABLE_AGENT_COMM_H
#define COACHABLE_AGENT_COMM_H

#include <string>
using namespace std;
#include "saymsglib/SayMsgEncoder.h"
#include "saymsglib/SayMsgDecoder.h"
#include "FocusStrategy.h"

// wyverns related includes: 
#include "WorldModel.h"



class CoachableAgentComm {

 public: 
  CoachableAgentComm(WorldModel *wm);
  ~CoachableAgentComm();

  /*
   * Return an encoded say message that we should say this cycle.
   *
   */
  string getSayMsg();

  /*
   * Return the number of the player we would like to listen to
   * (with "attentionto"). 
   */
  unsigned short getPlayerToFocusOn();

  /*
   * Input: an encoded say message received from player 'playernum'
   * Handle the message by decoding it and integrating the information
   * into our world model (or using the info in other ways). 
   */
  void handleReceivedMsg(char *msg, unsigned short playernum);


  SayMsgEncoder *getEncoder(){ return MyEncoder; }

 private:
  SayMsgEncoder *MyEncoder; // does all our msg encoding
  SayMsgDecoder *MyDecoder; // does our decoding
  WorldModel *WM;
  FocusStrategy *focusStrategy; // selects player to focus attention on.

  unsigned short attentionto; // player # of the player we're currently focussing on.
  Time timeLastReceived;     // time last msg was received. 

};



#endif

