/* Dongryeol Lee
 *
 * CoachablePlayer.h
 * Header file for CoachablePlayer class 
 */

#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <vector>
#include <memory>
#include <list>
#include "region.h"
#include "clangaction.h"
#include "clangutil.h"
#include "BasicPlayer.h"
#include "ActionBinding.h"
#include "ActionVisitor.h"
#include "../actionsgui/ActionsGUIInterface.h"


#ifndef COACHABLEPLAYER
#define COACHABLEPLAYER


using namespace rcss;
using namespace clang;

class CoachablePlayer : public BasicPlayer
{

 private:

  /*********** Private methods ****************/
  SoccerCommand searchForPlayer(const UNum::unum_t &player, bool ourSide);

  SoccerCommand scanField();

  AngDeg getSearchBallAngle();

  bool executeGoalieBehavior();


  UNum::unum_t selectRandomUNum(const UNumSet &uset);

  SoccerCommand selectPassMode(const UNum::unum_t &playerUNum);

  VecPosition getGoalieTargetPos();

  double DistanceForCycles(int numcycles);

  void guiSend(char *msg, char *fgcolor=NULL, char *bgcolor=NULL);

  // helper for dribble: 
  double evaluateLineDanger(VecPosition src, VecPosition dest);



  /********** Private member variables **********/

  /* The home region for the coachable player */
  Region *homeRegion;

  /* Offside line trap set for the player */
  double OffsideTrap;

  VarBindings myBindings; // bindings set in executeCommand procedure

  ActionBinding::SourceType actionsource; // stores the action sourcetype of the current action

  ActionsGUIInterface *gui; // pointer to our ActionsGUIInterface

  int LastTime;
  ActionVisitor::ActionType lastActionType;

 public:

  /* Tells whether the agent is in the home region */
  bool AmIHome();


  /*
   * This is an initialization function which is called from the Player 
   * class constructor
   */
  void InitCoachablePlayer(bool useGUI);
  
  ~CoachablePlayer()
    {
      /* make sure the region pointed by homeRegion is deleted to 
	 free memory held by the Region object */
      delete homeRegion;
    }

  /* The followings are the command modules for the Coachable agent */
  SoccerCommand passToUNumSet(const UNumSet &uset);

  SoccerCommand passToRegion(Region *reg);
  
  SoccerCommand shoot();
  
  SoccerCommand hold();

  SoccerCommand intercept_tackle();
  
  SoccerCommand interceptBall();

  SoccerCommand dribbleToRegion(Region *reg);

  SoccerCommand clear(Region *reg);

  SoccerCommand markUNumSet(const UNumSet &uset);
  
  SoccerCommand markLaneUNumSet(const UNumSet &uset);

  SoccerCommand markLaneRegion(Region *reg);

  SoccerCommand tackle(const UNumSet &uset);
  
  SoccerCommand posRegion(Region *reg);

  SoccerCommand setHomeRegion(Region *reg);

  SoccerCommand setOffsideTrap(Region *reg);

  SoccerCommand goHome();

  void executeCommand(const ActionBinding& act);

};

#endif
