/*
 * CommPicker - picks communication message units to put into a say message.
 *   Used by CoachableAgentComm to set up a message to send. 
 *
 *
 * Creation Date: 12/18/03
 * Author: John Davin
 *
 */

#ifndef COMM_PICKER_H
#define COMM_PICKER_H


#include "MySayMsgTypes.h"
#include "CoachableAgentComm.h"
#include "CommUtility.h"


class CommPicker {

 public: 
  CommPicker(CoachableAgentComm *agent);
  ~CommPicker();

  /*
   * Pick messages based on our heuristic. 
   * 'availableSize' is the number of characters we can use.
   */
  void pickMsgs(unsigned int availableSize);




 private: 
  class MsgPpair {
  public: 
    MsgPpair( MySayMsgTypes *m, float pr) : msg(m), p(pr),deleted(false) {}
    MsgPpair() : msg(NULL), p(0.),deleted(false) {}
    MySayMsgTypes *msg;
    float p; // probability
    bool deleted; // flag used by the pickMsgs algorithm
  };
  static bool SortMsgsByP(const MsgPpair &a, const MsgPpair &b);
  
  void add(MySayMsgTypes *msg, float p, vector<MsgPpair> *msgVector);

  void filter(vector<MsgPpair> *msgVector);

  void normalizeP(vector<MsgPpair> *msgsVector);

  void randomizeEquals(vector<MsgPpair> *msgsVector);

  void clearMsgs(vector<MsgPpair> *msgsVector);

  /********* Private Data: *************/
  CoachableAgentComm *AgentComm;

  CommUtility *utilityAccessor;

};


#endif







