/*
 * A Cond Visitor class which evaluates the condition and binds 
 * clang variables
 *
 * John Davin
 */


#ifndef COND_VISITOR_H
#define COND_VISITOR_H

#include "visitor.h"
#include "cond.h"
#include "hasa.h"
#include "config.h"
#include "RegionVisitor.h"
#include "VarBindings.h"


class CondVisitor : public Cond::Visitor
{
 private: 
  bool truthValue;
  VarBindingSet *myBindings;

  // helper function for PPos conditions: 
  bool playerInRegion(const UNum& player, bool ourside, 
		      const RegionVisitor& regionvisitor) const;

  int checkPlayersInRegion(const UNumSet& playerset, bool ourside, 
			   const RegionVisitor& regionvisitor, 
			   VarBindings *bindings,
			   int min, int max) const;

 public:

  // the cond visitor can be initialized with starting bindings
  CondVisitor(VarBindingSet *v)
    : myBindings(v)
    { truthValue = false; }

  ~CondVisitor();

  bool getEval(); // just return truthValue.

  VarBindingSet *getBindingSet() const { return myBindings; }


  /**** Basic boolean operations: ****/
  void visit( CondBool* cond );

  void visit( CondAnd* cond );

  void visit( CondOr* cond );

  void visit( CondNot* cond );


  /**** clang conditions: ****/
  void visit( CondPlayerPos* cond );
 
  void visit( CondBallPos* cond );
 
  void visit( CondBallOwner* cond );

  void visit( CondPlayMode* cond );

  void visit( CondNamed* cond );

  void visit( CondTime* cond ); 

  void visit( CondOppGoal* cond );

  void visit( CondOurGoal* cond );

  void visit( CondGoalDiff* cond );
 
  void visit( CondUNum* cond );


};


#endif // COND_VISITOR_H


