/* Dongryeol Lee
 *
 * Filter.C
 *
 * Implementation of the Filter class object used in the Action Picker
 *
 */

#include "ApplicabilityActionVisitor.h"
#include "ReductionActionVisitor.h"
#include "CompareActionVisitor.h"
#include "Filter.h"
#include "Logger.h"

void Filter::addDoNotAdvice(Action *addThisAction, const VarBindings &bindings,
			    ActionBinding::SourceType source)
{
  /* We need to "reduce" our Don't advice into pieces first */
  ReductionActionVisitor r(ActionBinding(addThisAction,bindings,source));

  /* Call accept on the action */
  addThisAction->accept(r);

  /* Retrieve the atomized action list */
  vector <ActionBinding> atomizedList = r.getAtomizedActions();

  /* Just append the forbidden list of atomized actions */
  doNotList.insert(doNotList.begin(), atomizedList.begin(),atomizedList.end());

}
  
bool Filter::isInDoNotList(const ActionBinding& abinding) const
{
  /* First, we define equality on two actions dealing with UNumSet as follows:
     if(actionType1 = actionType2 and UNumSet1 = UNumSet2)
     
     For now, defining equality on a regino type actions is hard, so it will
     be ignored */
  
  /* Create a CompareActionVisitor on the action we want to test */
  CompareActionVisitor compTo(abinding);


  /* Compare against each action stored in the Do Not List */
  for(unsigned int i = 0; i < doNotList.size(); i++) {

    /* First retrieve the action itself */
    Action *act = doNotList[i].theaction;
    
    /* Then, call accept on the act on the CompareActionVisitor */
    act->accept(compTo);

    /* If the comparison returns positive, then return true immediately */
    if(compTo.getResult()){
      actionlog(30) << "Filtering because in DoNot list "<<*abinding.theaction<<spades::ende;
      return true;
    }
  }

  /* If you are here, no match is found */
  return false;
}

/* Returns true if the advice fails the Applicability condition */
bool Filter::failsApplicability(const ActionBinding &abinding) const
{
  ApplicabilityActionVisitor a;

  /* Call accept on the visitor */
  (abinding.theaction)->accept(a);

  /* Return result */
  return !a.getApplicability();
}

/* Tells whether the given action binding should be filtered */
bool Filter::shouldBeFiltered(const ActionBinding &abinding) const
{
  /* Utilize short-circuit evaluation */
  return failsApplicability(abinding) || isInDoNotList(abinding);
}

void Filter::clear()
{
  doNotList.clear();
}
