/*
 * FocusStrategy - this is a simple set of classes that can be used to define 
 * different strategies for choosing what player to focus on for receiving 
 * communication messages. 
 * The main purpose of these classes is just for testing different focus strategies.
 *  When not testing, FocusNormal will be our only actual strategy. 
 * 
 * Creation Date: 1/1/04
 * Author: John Davin
 */


#ifndef FOCUS_STRATEGY_H
#define FOCUS_STRATEGY_H

#include "WorldModel.h"



class FocusStrategy {
 public: 
  FocusStrategy(WorldModel *wm) : WM(wm) {}

  /*
   * Return the number of the player ([1,11]) we want to focus on.
   * 
   */
  virtual unsigned short getDesiredFocus() = 0;

 protected:
  WorldModel *WM;
};



/*
 * Focus on ball owner, but if ball owner is far away from ball (need good conf 
 *  on ball and owner to determine this), then focus on player closest to ball.
 * If we are ball owner, focus on closest player?
 *
 */
class FocusNormal : public FocusStrategy {
 public:
  FocusNormal(WorldModel *wm) : FocusStrategy(wm) {}

  unsigned short getDesiredFocus();

};


/*
 * Focus on player 2. If we are player 2, focus on player 3.
 */
class FocusTest : public FocusStrategy {
 public:
  FocusTest(WorldModel *wm) : FocusStrategy(wm) {}

  unsigned short getDesiredFocus();

};



class FocusRandom : public FocusStrategy {
 public:
  FocusRandom(WorldModel *wm) : FocusStrategy(wm) {}

  unsigned short getDesiredFocus();

};


#endif
