/* Dongryeol Lee
 *
 * Level.C
 *
 * Implementation of the Level class
 *
 */


#include "Level.h"
#include "CompareActionVisitor.h"


#include "Logger.h"



/* Match world state here */
bool Level::matchWorldState() const
{
  /* Retrieve the number of world state predicates */
  unsigned int numWorldStatePred = worldStatePredicates.size();
  
  //actionlog(40) << "Matching world state" << spades::ende;


  /* Go through each world-state predicates and test truth values */
  for(unsigned int i = 0; i < numWorldStatePred; i++) {

    
    //actionlog(40) << i;


    /* Get the flag whether the current predicate is positive/negative */
    bool flag = worldStatePredicates[i]->getPositiveFlag();
    

    /* Tells whether the positive version of the predicate is satisfiable or
       not */
    bool isSatisfied = worldStatePredicates[i]->isSatisfiable();


    /* Terminate upon the first false predicate */
    if( flag && !isSatisfied || !flag && isSatisfied )
      {
	//actionlog(40) << ": failed " << flag << " " << isSatisfied << spades::ende;
	return false;
      }
    //actionlog(40) << ": passed" << spades::ende;
  }

  //actionlog(40) << "Passed world state" << spades::ende;

  /* If you are here, all the world state predicates are satisfied */
  return true;

}


/* Match action type */
bool Level::matchActionType(rcss::clang::Action *act) const
{
  /* Simply use the action visitor to extract the action type */
  ActionVisitor avisitor;

  act->accept(avisitor);

  /* And compare with the level action type */
  return actionType == avisitor.getActionType();
}


/* Match source type */
bool Level::matchSourceType(ActionBinding::SourceType s) const
{
  /* ANY source matches any advice except for Default actions. 
     Otherwise, match the source advice with the rule advice in a normal way */
  
  //actionlog(40) << "Matching source type" << spades::ende;
  //actionlog(40) << "    Comparing " << s << " with " << sourceType <<
  //spades::ende;


  //if( (sourceType == ActionBinding::ANY &&
  //s != ActionBinding::DEFAULT) || sourceType == s )
    //actionlog(40) << "Passed source type" << spades::ende;


  return ( (sourceType == ActionBinding::ANY && 
	    s != ActionBinding::DEFAULT)
	   || sourceType == s);
}


/* Match action specific properties here */
bool Level::matchAction(rcss::clang::Action *act, 
			const VarBindings &v ) const
{
  /* Retrieve the number of action predicates */
  unsigned int numActionPred = actionPredicates.size();
  
  //actionlog(40) << "Matching action" << spades::ende;


  /* Go through each action predicates and test truth values */
  for(unsigned int i = 0; i < numActionPred; i++) {

    //actionlog(40) << i << spades::ende;


    /* Get whether the current predicate is positive or negative */
    bool flag = actionPredicates[i]->getPositiveFlag();

    /* Determine whether the positive version of the current predicate is
       satisfiable or not */
    bool isSatisfied = actionPredicates[i]->isSatisfiable(act, v);

    
    /* Terminate up on the first false predicate */
    if( flag && !isSatisfied || !flag && isSatisfied ){
      actionlog(30)<< "Action failed Level::matchAction: "<<*act<<spades::ende;
      return false;
    }
    
    //actionlog(40) << "I'm still ok" << spades::ende;

  }

  //actionlog(40) << "You passed!!!" << spades::ende;


  /* Again, if you are here, then all action predicates are satisifed */
  return true;

}

/* Add world state predicates to the level */
Level Level::addWorldStatePredicate(WorldStatePredicate *p)
{
  worldStatePredicates.push_back(p);
  
  return *this;
}

/* Add action predicates to the level */
Level Level::addActionPredicate(ActionPredicate *p)
{
  actionPredicates.push_back(p);

  return *this;
}


/* Output to the standard output stream */
std::ostream& operator<<( std::ostream & os, const Level& l )
{ return l.print( os ); }

