/* Dongryeol Lee
 *
 * PointVisitor.h
 * 
 * Header file for PointVisitor Class
 */

#include "visitor.h"
#include "clangutil.h"
#include "region.h"
#include "Geometry.h"
#include "WorldModel.h"
#include "VarBindings.h"

#ifndef POINTVISITOR
#define POINTVISITOR

using namespace rcss::clang;

class PointVisitor: public Point::Visitor
{
 private:
  float globalX;
  float globalY;
  VarBindings myBindings;  // holds bindings for clang variables
  void init(); // constructor function

 public:
  
  PointVisitor(){ init(); }

  PointVisitor(VarBindings b)
    : myBindings(b)
    { init(); }

  VecPosition returnGlobalPosition();
  
  void visit(Point *p);

  void visit(PointSimple *p);
  
  void visit(PointRel *p);
  
  void visit(PointBall *p);
  
  void visit(PointPlayer *p);
  
  void visit(PointArith *p);

  /*
   * Return true if point visitor encountered variables. False otherwise.
   */
  bool hasVariables() { return !myBindings.empty(); }

  VarBindings* getBindings() { return &myBindings; }

};

#endif
