/* Dongryeol Lee
 *
 * ReductionActionVisitor.h
 *
 * This is the Reduction class that achieves two stages of "reductions" on an
 * action. All the methods are declared as static
 *
 * 1) Region -> UNumSet reduction: applies to passRegion and marklRegion action
 * 2) UNumSet of x UNum's -> x * UNumSet of 1 UNum's
 *    ("atomization" of multi UNum action into a single UNum action)
 *
 */


#ifndef REDUCTION_ACTION_VISITOR
#define REDUCTION_ACTION_VISITOR


#include <vector>
#include "visitor.h"
#include "clangaction.h"
#include "ActionBinding.h"
#include "VarBindings.h"


using namespace rcss::clang;
using namespace std;


class ReductionActionVisitor: public Action::Visitor
{

 public:

  /* Constructor */
  ReductionActionVisitor(const ActionBinding &abinding);

  void visit(ActPos *act);

  void visit(ActHome *act);
  
  void visit(ActBallToReg *act);
  
  void visit(ActBallToPlayer *act);
  
  void visit(ActMark *act);
  
  void visit(ActMarkLinePlayer *act);
  
  void visit(ActMarkLineReg *act);

  void visit(ActOffsidesLine *act);

  void visit(ActHetType *act);

  void visit(ActNamed *act);

  void visit(ActPassReg *act);

  void visit(ActPassUNum *act);

  void visit(ActDribble *act);

  void visit(ActClear *act);

  void visit(ActShoot *act);

  void visit(ActHold *act);

  void visit(ActIntercept *act);

  void visit(ActTackle *act);

 private:

  /* Store atomized ActionBindings here */
  vector <ActionBinding> atomizedActions;

  /* Variable bindings for this action */
  VarBindings bindings;
  
  /* Source of the actino advice */
  ActionBinding::SourceType sourceType;


 public:

  /* Retrieve the atomized actions after visiting */
  const vector <ActionBinding> getAtomizedActions() const
    { return atomizedActions; }
};


#endif
