/* Dongryeol Lee
 *
 * ScoringActionVisitor.h
 *
 * This is the header file for the implementation of the ScorintActionVisitor
 * class. Using the Action Visitor class methods, an appropriate scoring
 * function will be called for each action.
 *
 */


#ifndef SCORING_ACTION_VISITOR
#define SCORING_ACTION_VISITOR


#include "ActionBinding.h"
#include "clangaction.h"


using namespace rcss::clang;


class ScoringActionVisitor: public Action::Visitor
{
 public:

  /* Constructor */
  ScoringActionVisitor(ActionBinding::SourceType st) 
    { 
      /* Initialize the score to 0 */
      score = 0.0;

      /* Get the source of the advice */
      sourceType = st;

    }


  double getScore() const { return score; }

  void visit(ActPos *act);

  void visit(ActHome *act);
  
  void visit(ActBallToReg *act);
  
  void visit(ActBallToPlayer *act);
  
  void visit(ActMark *act);
  
  void visit(ActMarkLinePlayer *act);
  
  void visit(ActMarkLineReg *act);

  void visit(ActOffsidesLine *act);

  void visit(ActHetType *act);

  void visit(ActNamed *act);

  void visit(ActPassReg *act);

  void visit(ActPassUNum *act);

  void visit(ActDribble *act);

  void visit(ActClear *act);

  void visit(ActShoot *act);

  void visit(ActHold *act);

  void visit(ActIntercept *act);

  void visit(ActTackle *act);

 private:
  
  double score;

  ActionBinding::SourceType sourceType;

};


#endif
