/*
 * An implementation of the WorldModelInterface abstract functions. 
 * 
 *
 * John Davin
 */


#include "WorldModelAccessor.h"
#include "config.h"

using namespace rcss::clang;

/*
 * Return the global position of the agent specified by num and team ourside
 * 
 */
VecPosition WorldModelAccessor::getGlobalPosition(UNum::unum_t num, bool ourside){

  ObjectT player = getObjectTFromUNum(num, ourside);

  return trilearn_wm->getGlobalPosition(player);
}


/* 
 * Returns the current PlayMode
 * This accesses the worldmodel, but the trilearn code has their own type, called PlayModeT. 
 * The types don't match up exactly with the soccerserver PlayMode type, so I tried to match
 * up whatever I could figure out (but there are more PlayModeT types than PlayMode types). 
 */
rcss::clang::PlayMode WorldModelAccessor::getPlayMode() {

  // leftside is true if we're a left side player (and false if right side). 
  bool leftside = TeamIsLeftSide();

  switch( trilearn_wm->getPlayMode() )
    {
    case PM_BEFORE_KICK_OFF:       
      return rcss::clang::BeforeKickOff;
    case PM_KICK_OFF_LEFT:         
      return (leftside ? rcss::clang::KickOff_Our : rcss::clang::KickOff_Opp);
    case PM_KICK_OFF_RIGHT:        
      return (leftside ? rcss::clang::KickOff_Opp : rcss::clang::KickOff_Our);
    case PM_KICK_IN_LEFT:          
      return (leftside ? rcss::clang::KickIn_Our : rcss::clang::KickIn_Opp);
    case PM_KICK_IN_RIGHT:         
      return (leftside ? rcss::clang::KickIn_Opp : rcss::clang::KickIn_Our);
    case PM_CORNER_KICK_LEFT:      
      return (leftside ? rcss::clang::CornerKick_Our : rcss::clang::CornerKick_Opp);
    case PM_CORNER_KICK_RIGHT:     
      return (leftside ? rcss::clang::CornerKick_Opp : rcss::clang::CornerKick_Our);
    case PM_GOAL_KICK_LEFT:        
      return (leftside ? rcss::clang::GoalKick_Our : rcss::clang::GoalKick_Opp);
    case PM_GOAL_KICK_RIGHT:       
      return (leftside ? rcss::clang::GoalKick_Opp : rcss::clang::GoalKick_Our);
    case PM_GOAL_LEFT:             
      return (leftside ? rcss::clang::AfterGoal_Our : rcss::clang::AfterGoal_Opp); // is this correct?
    case PM_GOAL_RIGHT:            
      return (leftside ? rcss::clang::AfterGoal_Opp : rcss::clang::AfterGoal_Our);
    case PM_FREE_KICK_LEFT:        
      return (leftside ? rcss::clang::FreeKick_Our : rcss::clang::FreeKick_Opp);
    case PM_FREE_KICK_RIGHT:       
      return (leftside ? rcss::clang::FreeKick_Opp : rcss::clang::FreeKick_Our);
    case PM_OFFSIDE_LEFT:         
      return (leftside ? rcss::clang::GoalieCatch_Our : rcss::clang::GoalieCatch_Opp); // is this correct?
    case PM_OFFSIDE_RIGHT:         
      return (leftside ? rcss::clang::GoalieCatch_Opp : rcss::clang::GoalieCatch_Our);

    case PM_PLAY_ON:               return rcss::clang::PlayOn;
      //case PM_FROZEN:                return "play_off"; // what does this correspond to?
    case PM_QUIT:                  return rcss::clang::TimeOver;

      // I don't know what these are for: 
    case PM_FREE_KICK_FAULT_LEFT:  

    case PM_FREE_KICK_FAULT_RIGHT: 
	
    case PM_BACK_PASS_LEFT:        
    case PM_BACK_PASS_RIGHT:       

    case PM_ILLEGAL:
   
    default:                       
      //PR(9, "[WorldModelAccessor:GetPlayMode]: Warning! Unhandled PlayModeT.\n");

      //cout << "Undefined PlayMode!!\n";

      return rcss::clang::PM_None;
    }

}

/*
 * Helper function to getBallOwner
 * Converts from trilearn's ObjectT type to a player unum_t type. 
 * The input ObjectT must be a player object type. 
 */
UNum::unum_t WorldModelAccessor::getUNumFromObjectT(ObjectT trilearnobj){

  switch(trilearnobj){

  case OBJECT_TEAMMATE_1:
  case OBJECT_OPPONENT_1:   return UNum::u1;

  case OBJECT_TEAMMATE_2:
  case OBJECT_OPPONENT_2:   return UNum::u2;

  case OBJECT_TEAMMATE_3:
  case OBJECT_OPPONENT_3:   return UNum::u3;

  case OBJECT_TEAMMATE_4:
  case OBJECT_OPPONENT_4:   return UNum::u4;

  case OBJECT_TEAMMATE_5:
  case OBJECT_OPPONENT_5:   return UNum::u5;

  case OBJECT_TEAMMATE_6:
  case OBJECT_OPPONENT_6:   return UNum::u6;

  case OBJECT_TEAMMATE_7: 
  case OBJECT_OPPONENT_7:   return UNum::u7;

  case OBJECT_TEAMMATE_8:
  case OBJECT_OPPONENT_8:   return UNum::u8;

  case OBJECT_TEAMMATE_9:
  case OBJECT_OPPONENT_9:   return UNum::u9;

  case OBJECT_TEAMMATE_10:
  case OBJECT_OPPONENT_10:   return UNum::u10;

  case OBJECT_TEAMMATE_11: 
  case OBJECT_OPPONENT_11:   return UNum::u11;

  case OBJECT_PLAYER_UNKNOWN:  // no ball owner yet. 
                             return UNum::uAll;

  default:
    PR(9, "[WorldModelAccessor::getUNumFromObjectT]: ERROR: input object type was not a player.\n");
    return UNum::uAll;
  }
}

ObjectT WorldModelAccessor::getObjectTFromUNum(rcss::clang::UNum::unum_t
					       playerUNum, bool ourside)
{
  switch(playerUNum){
  case UNum::u1:   
    return (ourside ? (OBJECT_TEAMMATE_1) : 
	    (OBJECT_OPPONENT_1));
    break;
  case UNum::u2:  
    return (ourside ? (OBJECT_TEAMMATE_2) : 
	    (OBJECT_OPPONENT_2));
    break;
  case UNum::u3: 
    return (ourside ? (OBJECT_TEAMMATE_3) : 
	    (OBJECT_OPPONENT_3));
    break;
  case UNum::u4: 
    return (ourside ? (OBJECT_TEAMMATE_4) : 
	    (OBJECT_OPPONENT_4));
    break;
  case UNum::u5: 
    return (ourside ? (OBJECT_TEAMMATE_5) : 
	    (OBJECT_OPPONENT_5));
    break;
  case UNum::u6: 
    return (ourside ? (OBJECT_TEAMMATE_6) : 
	    (OBJECT_OPPONENT_6));
    break;
  case UNum::u7: 
    return (ourside ? (OBJECT_TEAMMATE_7) : 
	    (OBJECT_OPPONENT_7));
    break;
  case UNum::u8: 
    return (ourside ? (OBJECT_TEAMMATE_8) : 
	    (OBJECT_OPPONENT_8));
    break;
  case UNum::u9: 
    return (ourside ? (OBJECT_TEAMMATE_9) : 
	    (OBJECT_OPPONENT_9));
    break;
  case UNum::u10: 
    return (ourside ? (OBJECT_TEAMMATE_10) : 
	    (OBJECT_OPPONENT_10));
    break;
  case UNum::u11:  
    return (ourside ? (OBJECT_TEAMMATE_11) : 
	    (OBJECT_OPPONENT_11));
    break;

  case UNum::uAll: 
  default:    
    PR(9, "[WorldModelAccessor:getObjectTFromUNum]: Warning! Unhandled UNum\n");
    return (OBJECT_TEAMMATE_1);
    //assert(0);
    break;

  }
}


bool WorldModelAccessor::getBallOwner(UNum::unum_t *player){

  ObjectT objplayer = trilearn_wm->getBallOwner();
  *player = getUNumFromObjectT(objplayer);
  
  if(SoccerTypes::isTeammate(objplayer))
    return true;
  else if(SoccerTypes::isOpponent(objplayer))
    return false;
  else{
    PR(9, "[WorldModelAccessor]: Warning: getBallOwner called without ball owner set yet.");
    return false;
  }
}

bool WorldModelAccessor::playerConfidenceGood(rcss::clang::UNum::unum_t
					      playerUNum, bool ourSide)
{
  
  if(ourSide)
    {
      /* The trilearn world model returns false when the object is ourself
	 I don't know hwy this is, and I think it's wrong but it seems dangerous
	 to make the change there. Therefore, we make the change here to look
	 at our confidence */
      if (playerUNum == getMyNum())
	return true; //pfr: this is kind of a hack; I can't figure ou how to get our confidence from the world model
      else
	return trilearn_wm->isConfidenceGood((ObjectT) 
					     (OBJECT_TEAMMATE_1 + 
					      (int) playerUNum - 1));
    }
  else
    return trilearn_wm->isConfidenceGood((ObjectT)
					 (OBJECT_OPPONENT_1 + 
					  (int) playerUNum - 1));
  
}



/* Returns true if the player with the UNum's position is visible by
   the agent */
bool WorldModelAccessor::playerVisibilityGood(UNum::unum_t playerUNum, 
					      bool ourSide)
{
  return trilearn_wm->isVisible(getObjectTFromUNum(playerUNum, ourSide));
}


VecPosition WorldModelAccessor::predictPosAfterNrCycles(UNum::unum_t u,
							bool ourSide,
							int iCycles)
{
  /* Convert UNum into ObjectT objects */
  ObjectT o = getObjectTFromUNum(u, ourSide);

  return trilearn_wm->predictPosAfterNrCycles(o, iCycles);

}


/*! This method returns the stamina information of the agent
    \return Stamina stamina of the agent */
Stamina WorldModelAccessor::getAgentStamina( ) const
{
  return trilearn_wm->getAgentStamina();
}

/* Returns how tired the agent is */
TiredNessT WorldModelAccessor::getAgentTiredNess( ) const
{
  return trilearn_wm->getAgentTiredNess();
}

bool WorldModelAccessor::matchGoalie(const UNum::unum_t &u,
				     bool ourSide)
{
  return getObjectTFromUNum(u, ourSide) == trilearn_wm->getOwnGoalieType();
}

bool WorldModelAccessor::FastestToBall() const
{
  int iTmp;
  
  return  (   trilearn_wm->getFastestInSetTo
	      ( OBJECT_SET_TEAMMATES, OBJECT_BALL, &iTmp )
	      == trilearn_wm->getAgentObjectType()   );  
}
