/*
    Coachable Agent communication encoding and decoding library
    Copyright (C) 2004, John Davin, Carnegie Mellon University

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/*
 * SampleSayMsgTypes.h - Defines subclasses of the SayMsgTypes in order to 
 *  define our process() functions (which can add data to the world model)
 *
 * Creation Date: 11/18/03
 * Author: John Davin
 *
 *
 */

#ifndef SAMPLE_SAY_MSG_TYPES_H
#define SAMPLE_SAY_MSG_TYPES_H


#include "SayMsgTypes.h"


/*
 * OurPos member variables: 
 * float X - our position X value 
 * float Y - our position Y value
  */
class MyOurPos : public OurPos {

 public:   

  void process(){
    /* add X and Y positions of sending player to our world model? */
    
    cout << "MyOurPos: playernum="<<SenderPlayerNum<<" X="<<X<<" Y="<<Y<<endl;

  }

};

/*
 * BallPos member variables: 
 * float X - ball position X value 
 * float Y - ball position Y value
 * unsigned short cycleno - number of cycles since saw ball
 */
class MyBallPos : public BallPos {

 public: 
  void process(){
    
    cout << "MyBallPos: playernum="<<SenderPlayerNum<<" reports ball at ("<<X
	 <<","<<Y<<") with confidence "<<cycleno<<" cycles."<<endl;

  }

};

/*
 * BallVel member variables: 
 * float dX - ball velocity X value 
 * float dY - ball velocity Y value
 * unsigned short cycleno - number of cycles since saw ball
 */
class MyBallVel : public BallVel {

 public: 
  void process(){
    
    cout << "MyBallVel: playernum="<<SenderPlayerNum<<" reports ball at velocity ("
	 << dX<<","<<dY<<") with cycles="<<cycleno<<endl;
  }

};


/*
 * WeHaveBall member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyWeHaveBall : public WeHaveBall {

 public: 
  void process(){

    if(playernum == 0)
      cout << "MyWeHaveBall: playernum="<<SenderPlayerNum
	   <<" reports that we have ball (playernum=0)."<<endl;
    else
      cout << "MyWeHaveBall: playernum="<<SenderPlayerNum<<" reports our player "
	 <<playernum<<" has ball."<<endl;
  }

};


/*
 * OppHasBall member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyOppHasBall : public OppHasBall {

 public: 
  void process(){
    if(playernum == 0)
      cout << "MyOppHasBall: playernum="<<SenderPlayerNum
	   <<" reports opp team has ball (playernum=0)"<<endl;
    else
      cout << "MyOppHasBall: playernum="<<SenderPlayerNum<<" reports opp player "<<playernum<<
	" has ball."<<endl;
  }

};

/*
 * PassToPlayer member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyPassToPlayer : public PassToPlayer {

 public: 
  void process(){
    
    cout << "MyPassToPlayer: playernum="<<SenderPlayerNum<<
      " is passing to player "<<playernum<<endl;
  }

};

/*
 * PassToCoord member variables: 
 * float X - X coordinate of position we're passing to.
 * float Y - Y coordinate of position we're passing to. 
 */
class MyPassToCoord : public PassToCoord {

 public: 
  void process(){
    
    cout << "MyPassToCoord: playernum="<<SenderPlayerNum<<" is passing to ("
	 <<X<<","<<Y<<")"<<endl;
  }

};


/*
 * WantPass member variables:
 * none
 */
class MyWantPass : public WantPass {

 public: 
  void process(){
    
    cout << "MyWantPass: playernum="<<SenderPlayerNum<<" wants pass."<<endl;

  }

};


/*
 * OppPos member variables: 
 * float X - opponent position X value 
 * float Y - opponent position Y value
 * unsigned short cycleno - number of cycles since saw player
 * unsigned short playernum - # of player at pos X,Y
 */
class MyOppPos : public OppPos {

 public: 
  void process(){
    
    cout << "MyOppPos: playernum="<<SenderPlayerNum<<" reports opponent #"<<playernum
	 <<" at ("<<X<<","<<Y<<") with confidence "<<cycleno<<" cycles."<<endl;
  }

};



/*
 * TeammatePos member variables: 
 * float X - teammate position X value 
 * float Y - teammate position Y value
 * unsigned short cycleno - number of cycles since saw player
 * unsigned short playernum - # of player at pos X,Y
 */
class MyTeammatePos : public TeammatePos {

 public: 
  void process(){
    
    cout << "MyTeammatePos: playernum="<<SenderPlayerNum<<" reports teammate #"<<playernum
	 <<" at ("<<X<<","<<Y<<") with confidence "<<cycleno<<" cycles."<<endl;
  }

};



#endif
